@@ -33,12 +33,18 @@ void ConfigurationTest::cleanupTestCase() { }
 
 void ConfigurationTest::init() {
     QFile::remove(CONF_FILE);
+    QDir(CONF_DIR).removeRecursively();
+    QDir().mkdir(CONF_DIR);
+    QDir(SYS_CONF_DIR).removeRecursively();
+    QDir().mkdir(SYS_CONF_DIR);
     QFile::remove(CONF_FILE_COPY);
     config = new TestConfig;
 }
 
 void ConfigurationTest::cleanup() {
     QFile::remove(CONF_FILE);
+    QDir(CONF_DIR).removeRecursively();
+    QDir(SYS_CONF_DIR).removeRecursively();
     QFile::remove(CONF_FILE_COPY);
     if (config)
         delete config;
@@ -118,6 +124,8 @@ void ConfigurationTest::LineChanges() {
 }
 
 void ConfigurationTest::CustomEnum() {
+
+    QTest::qWait(2000);
     QFile confFile(CONF_FILE);
     confFile.open(QIODevice::WriteOnly | QIODevice::Truncate);
     confFile.write("Custom=bar\n");
@@ -152,10 +160,50 @@ void ConfigurationTest::RightOnInit() {
     QVERIFY(config->Custom.get() == TestConfig::BAZ);
 }
 
+
+void ConfigurationTest::RightOnInitDir() {
+    delete config;
+
+    QFile confFileA(SYS_CONF_DIR+QStringLiteral("/0001A"));
+    confFileA.open(QIODevice::WriteOnly | QIODevice::Truncate);
+    confFileA.write("Custom=Foo\n"); //overriden by B
+    confFileA.write("Boolean=false\n");
+    confFileA.close();
+
+    QFile confFileB(CONF_DIR+QStringLiteral("/0001A"));
+    confFileB.open(QIODevice::WriteOnly | QIODevice::Truncate);
+    confFileB.write("String=a\n"); //overriden by C
+    confFileB.write("Custom=Bar\n");
+    confFileB.write("StringList=a,b,c\n");
+    confFileB.write("Int=1111111\n"); //this is set in this config file but overriden in CONF_FILE
+    confFileB.close();
+
+    QFile confFileC(CONF_DIR+QStringLiteral("/0001B"));
+    confFileC.open(QIODevice::WriteOnly | QIODevice::Truncate);
+    confFileC.write("String=b\n");
+    confFileC.write("Int=1111111\n"); //overriden in CONF_FILE
+    confFileC.close();
+
+    QFile confFileMain(CONF_FILE);
+    confFileMain.open(QIODevice::WriteOnly | QIODevice::Truncate);
+    confFileMain.write("Int=99999\n");
+    confFileMain.close();
+    confFileB.close();
+
+    config = new TestConfig;
+    QVERIFY(config->StringList.get() == QStringList({QStringLiteral("a"), QStringLiteral("b"), QStringLiteral("c")}));
+    QVERIFY(config->String.get() == QStringLiteral("b"));
+    QVERIFY(config->Int.get() == 99999);
+    QVERIFY(config->Custom.get() == TestConfig::BAR);
+    QVERIFY(config->Boolean.get() == false);
+}
+
 void ConfigurationTest::FileChanged()
 {
     QVERIFY(config->String.get() == QStringLiteral("Test Variable Initial String"));
 
+    QTest::qWait(2000);
+
     //test from no file to a file
     QFile confFile(CONF_FILE);
     confFile.open(QIODevice::WriteOnly | QIODevice::Truncate);
@@ -175,6 +223,25 @@ void ConfigurationTest::FileChanged()
 
     config->load();
     QVERIFY(config->String.get() == QStringLiteral("b"));
+
+    QTest::qWait(2000);
+
+    //add file to conf dir
+    QFile confFileA(CONF_DIR+QStringLiteral("/0001A"));
+    confFileA.open(QIODevice::WriteOnly | QIODevice::Truncate);
+    confFileA.write("Int=1111111\n"); //this is set in this config file but overriden in CONF_FILE
+    confFileA.close();
+    config->load();
+    QVERIFY(config->Int.get() ==1111111);
+
+    QTest::qWait(2000);
+    //modify existing file in conf dir
+
+    confFileA.open(QIODevice::WriteOnly | QIODevice::Truncate);
+    confFileA.write("Int=222222\n"); //this is set in this config file but overriden in CONF_FILE
+    confFileA.close();
+    config->load();
+    QVERIFY(config->Int.get() == 222222);
 }
 
 #include "moc_ConfigurationTest.cpp"
